(* ::Package:: *)

BeginPackage["Cookbook`Dictionary`"];

Dictionary::usage = "A Dictionary associates keys with a list of values";

makeDictionary::usage = "Creates a new Dictionary";

destroyDictionary::usage = "Destroys a dictionary and all contents";

dictName::usage = "Returns the generated unique name of the dictionary";

dictStore::usage = "Associate a value with key adding it to existing values";

dictReplace::usage = "Associate a value with key removing any duplicate values";

dictLookup::usage = "";

dictKeys::usage = "Returns a list of all keys";

dictHasKeyQ::usage = "Returns True if key exists";
 
dictKeyValuePairs::usage = "Return list of all {key,values} in dictionary";

dictClone::usage = "Clones a dictionary by creating a new dictionary with same contents";
 
Begin["`Private`"];

makeDictionary[] := 
	Module[{name},
		name = Unique["dict"] ;
		Evaluate[name][k_] := {};
		Dictionary[name]
	]

destroyDictionary[Dictionary[name_,___]] := 
	If[ValueQ[name[_]],Remove[name];True,False]

dictName[Dictionary[name_,___]] :=name

dictStore[dict_Dictionary,key_,value_] := 
	Module[{d =  dictName[dict]},
			d[key] = Prepend[d[key],value]
	]

dictReplace[dict_Dictionary,key_,value_] := 
	Module[{d =  dictName[dict]},
			d[key] = Union[d[key], {value}]
	]

dictRemove[dict_Dictionary,key_,value_] := 
	Module[{d =  dictName[dict]},
			d[key] = Complement[d[key],{value}]
	]

dictClear[Dictionary[name_,___]] := 
	If[ValueQ[name[_]],
		Clear[name];Evaluate[name][k_] := {}; True,
		False]

dictLookup[Dictionary[name_,___],key_] := name[key]

dictHasKeyQ[Dictionary[name_,___],key_] := ValueQ[name[key]]

dictKeys[dict_Dictionary] :=
	Most[DownValues[Evaluate[dictName[dict]]]] /. 
		HoldPattern[a_:> _List] :> a[[1,1]]

dictKeyValuePairs[dict_Dictionary] :=
	Most[DownValues[Evaluate[dictName[dict]]]] /. 
		HoldPattern[a_:> values_List] :> {a[[1,1]],values}

End[];

EndPackage[];
