(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     76330,       2205]
NotebookOptionsPosition[     55281,       1674]
NotebookOutlinePosition[     67607,       1922]
CellTagsIndexPosition[     67328,       1912]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[TextData[{
 "CHAPTER ",
 
 CounterBox["ChapterLabel"]
}], "ChapterLabel",
 CounterAssignments->{{"ChapterLabel", 5}},
 CellTags->"chStrings"],

Cell["String and Text Processing", "ChapterTitle"],

Cell["\<\
Someone will call
Something will fall
And smash on the floor
Without reading the text
Know what comes next
Seen it before
And it\[CloseCurlyQuote]s painful
Things must change
We must rearrange them
Or we\[CloseCurlyQuote]ll have to estrange them
All that I\[CloseCurlyQuote]m saying
The game\[CloseCurlyQuote]s not worth playing\[LineSeparator]Over and over \
again\
\>", "Epigraph",
 CellChangeTimes->{{3.4701598900153923`*^9, 3.470159904117351*^9}, {
  3.4705166289690437`*^9, 3.470516649621358*^9}, {3.475586795201412*^9, 
  3.475586805136717*^9}}],

Cell["\<\
Depeche Mode, \[OpenCurlyDoubleQuote]The Sun and the Rainfall\
\[CloseCurlyDoubleQuote]\
\>", "EpigraphAuthor",
 CellChangeTimes->{{3.4701597508499413`*^9, 3.470159777768581*^9}, 
   3.475586810216689*^9}],

Cell[CellGroupData[{

Cell[TextData[{
 
 CounterBox["ChapterLabel"],
 ".",
 
 CounterBox["Heading1"],
 "\[ThickSpace]\[ThickSpace]",
 "Introduction"
}], "Heading1",
 CellTags->"IntroductionCh5"],

Cell["\<\
Users who come to Mathematica for its superior mathematical capabilities are \
pleasantly surprised to find strong abilities in programming areas outside of \
mathematics proper. This is certainly true in the area of textual and string \
processing. Mathematica\[CloseCurlyQuote]s rich library of functions for \
string and structured text manipulation rivals Java, Perl, or any other \
modern language you can tie a string around. \
\>", "Text",
 CellChangeTimes->{{3.470159917890094*^9, 3.47015991857397*^9}, {
   3.470159950291054*^9, 3.4701599602513933`*^9}, 3.470160001373643*^9}],

Cell["\<\
The sections in this introduction provide information on some of the basic \
tools of strings and string manipulation. \
\>", "Text",
 CellChangeTimes->{
  3.4657710234586*^9, {3.470160010517082*^9, 3.470160011010084*^9}}],

Cell[CellGroupData[{

Cell["Characters and Character Encodings", "Heading2"],

Cell[TextData[{
 "Mathematica uses ",
 StyleBox["Unicode", "Techical Italic"],
 " internally, but externally (e.g., when saving a notebook) it uses ASCII \
codes, encoding non-ASCII characters in a special form."
}], "Text",
 CellChangeTimes->{
  3.4657710299482*^9, {3.470160062359996*^9, 3.470160102833699*^9}}],

Cell[TextData[{
 StyleBox["For example, lowercase Greek letters and other non-ASCI",
  FontTracking->"Plain"],
 StyleBox["I characters are encoded us",
  FontTracking->"Extended"],
 "ing backslash-bracketed character names (",
 StyleBox["\\[", "Literal"],
 StyleBox["name", "LiteralItalic"],
 StyleBox["]", "Literal"],
 ")."
}], "Text",
 CellChangeTimes->{3.4701601450607347`*^9, 3.4701603185625257`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"alpha", " ", "=", " ", "\"\<\[Alpha]\>\""}]], "Input",
 CellLabel->"In[1]:="],

Cell[BoxData["\<\"\[Alpha]\"\>"], "Output",
 CellChangeTimes->{3.4657716095861998`*^9, 3.4658598439955997`*^9, 
  3.468597740807213*^9, 3.468597864143873*^9, 3.470067176884511*^9},
 CellLabel->"Out[1]="]
}, Open  ]],

Cell[TextData[{
 "The function ",
 StyleBox["ToString", "Literal"],
 " will translate strings using different encoding schemes."
}], "Text",
 CellChangeTimes->{3.4657710503218*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToString", "[", 
  RowBox[{"alpha", ",", " ", 
   RowBox[{"CharacterEncoding", "\[Rule]", " ", "\"\<ASCII\>\""}]}], 
  "]"}]], "Input",
 CellLabel->"In[2]:="],

Cell[BoxData["\<\"\\\\[Alpha]\"\>"], "Output",
 CellChangeTimes->{3.4657716100074*^9, 3.4658598440186*^9, 
  3.4685977408459044`*^9, 3.468597864180716*^9, 3.4700671769406443`*^9},
 CellLabel->"Out[2]="]
}, Open  ]],

Cell[TextData[{
 StyleBox["The default character encoding used by Mathematica is stored in",
  FontTracking->"SemiCondensed"],
 " ",
 StyleBox["$CharacterEncoding", "Literal"],
 ", and the native character encoding of the underlying operating system \
Mathematica is running is stored in ",
 StyleBox["$SystemCharacterEncoding", "Literal"],
 ". All available encodings are stored in ",
 StyleBox["$CharacterEncodings", "Literal"],
 "."
}], "Text",
 CellChangeTimes->{{3.4701603394680767`*^9, 3.470160341119953*^9}}],

Cell[CellGroupData[{

Cell[BoxData["$CharacterEncoding"], "Input",
 CellLabel->"In[3]:="],

Cell[BoxData["\<\"UTF-8\"\>"], "Output",
 CellChangeTimes->{3.4657716101633997`*^9, 3.4658598440425997`*^9, 
  3.468597740881606*^9, 3.468597864226738*^9, 3.470067176984827*^9},
 CellLabel->"Out[3]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["$SystemCharacterEncoding"], "Input",
 CellLabel->"In[4]:="],

Cell[BoxData["\<\"UTF-8\"\>"], "Output",
 CellChangeTimes->{3.4657716102258*^9, 3.4658598440646*^9, 
  3.468597740926589*^9, 3.4685978642771606`*^9, 3.47006717705127*^9},
 CellLabel->"Out[4]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Partition", "[", 
   RowBox[{"$CharacterEncodings", ",", "4"}], "]"}], "//", 
  "TableForm"}]], "Input",
 CellLabel->"In[5]:="],

Cell[BoxData[
 TagBox[GridBox[{
    {"\<\"AdobeStandard\"\>", "\<\"ASCII\"\>", "\<\"CP936\"\>", \
"\<\"CP949\"\>"},
    {"\<\"CP950\"\>", "\<\"Custom\"\>", "\<\"EUC-JP\"\>", "\<\"EUC\"\>"},
    {"\<\"IBM-850\"\>", "\<\"ISO10646-1\"\>", "\<\"ISO8859-10\"\>", \
"\<\"ISO8859-11\"\>"},
    {"\<\"ISO8859-13\"\>", "\<\"ISO8859-14\"\>", "\<\"ISO8859-15\"\>", \
"\<\"ISO8859-16\"\>"},
    {"\<\"ISO8859-1\"\>", "\<\"ISO8859-2\"\>", "\<\"ISO8859-3\"\>", \
"\<\"ISO8859-4\"\>"},
    {"\<\"ISO8859-5\"\>", "\<\"ISO8859-6\"\>", "\<\"ISO8859-7\"\>", \
"\<\"ISO8859-8\"\>"},
    {"\<\"ISO8859-9\"\>", "\<\"ISOLatin1\"\>", "\<\"ISOLatin2\"\>", \
"\<\"ISOLatin3\"\>"},
    {"\<\"ISOLatin4\"\>", "\<\"ISOLatinCyrillic\"\>", "\<\"Klingon\"\>", \
"\<\"koi8-r\"\>"},
    {"\<\"MacintoshArabic\"\>", "\<\"MacintoshChineseSimplified\"\>", \
"\<\"MacintoshChineseTraditional\"\>", "\<\"MacintoshCroatian\"\>"},
    {"\<\"MacintoshCyrillic\"\>", "\<\"MacintoshGreek\"\>", \
"\<\"MacintoshHebrew\"\>", "\<\"MacintoshIcelandic\"\>"},
    {"\<\"MacintoshKorean\"\>", "\<\"MacintoshNonCyrillicSlavic\"\>", \
"\<\"MacintoshRomanian\"\>", "\<\"MacintoshRoman\"\>"},
    {"\<\"MacintoshThai\"\>", "\<\"MacintoshTurkish\"\>", \
"\<\"MacintoshUkrainian\"\>", "\<\"Math1\"\>"},
    {"\<\"Math2\"\>", "\<\"Math3\"\>", "\<\"Math4\"\>", "\<\"Math5\"\>"},
    {"\<\"Mathematica1\"\>", "\<\"Mathematica2\"\>", "\<\"Mathematica3\"\>", \
"\<\"Mathematica4\"\>"},
    {"\<\"Mathematica5\"\>", "\<\"Mathematica6\"\>", "\<\"Mathematica7\"\>", \
"\<\"PrintableASCII\"\>"},
    {"\<\"ShiftJIS\"\>", "\<\"Symbol\"\>", "\<\"Unicode\"\>", \
"\<\"UTF8\"\>"},
    {"\<\"WindowsANSI\"\>", "\<\"WindowsBaltic\"\>", \
"\<\"WindowsCyrillic\"\>", "\<\"WindowsEastEurope\"\>"},
    {"\<\"WindowsGreek\"\>", "\<\"WindowsThai\"\>", "\<\"WindowsTurkish\"\>", \
"\<\"ZapfDingbats\"\>"}
   },
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[0.7]}, 
       Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}, "RowsIndexed" -> {}}],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.4657716104286003`*^9, 3.4658598440886*^9, 
  3.468597740965328*^9, 3.4685978643439837`*^9, 3.4700671773060637`*^9},
 GridBoxOptions->{BaseStyle->{Magnification -> 0.8299999999999998}},
 CellLabel->"Out[5]//TableForm="]
}, Open  ]],

Cell["Notice how UTF-8 needs two bytes to display alpha.", "Text",
 CellChangeTimes->{
  3.4701603724625053`*^9, {3.471623477491583*^9, 3.471623482607757*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToString", "[", 
  RowBox[{"alpha", ",", " ", 
   RowBox[{"CharacterEncoding", "\[Rule]", " ", "\"\<UTF8\>\""}]}], 
  "]"}]], "Input",
 CellLabel->"In[6]:="],

Cell[BoxData["\<\"\[CapitalIHat]\[PlusMinus]\"\>"], "Output",
 CellChangeTimes->{3.4657716105690002`*^9, 3.4658598441126003`*^9, 
  3.468597741010457*^9, 3.468597864393433*^9, 3.470067177354908*^9},
 CellLabel->"Out[6]="]
}, Open  ]],

Cell[TextData[{
 StyleBox["ToCharacterCode", "Literal"],
 " gives the numerical representation."
}], "Text",
 CellChangeTimes->{3.4657710845950003`*^9, 3.470160388181926*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToCharacterCode", "[", 
  RowBox[{"ToString", "[", 
   RowBox[{"alpha", ",", " ", 
    RowBox[{"CharacterEncoding", "\[Rule]", " ", "\"\<UTF8\>\""}]}], "]"}], 
  "]"}]], "Input",
 CellLabel->"In[7]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"206", ",", "177"}], "}"}]], "Output",
 CellChangeTimes->{3.4657716110058002`*^9, 3.4658598441366*^9, 
  3.468597741060193*^9, 3.468597864443883*^9, 3.470067177401924*^9},
 CellLabel->"Out[7]="]
}, Open  ]],

Cell[TextData[{
 "You can map from character codes back to characters using ",
 StyleBox["FromCharacterCode[]", "Literal"],
 "."
}], "Text", "ProductionPageBreak"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FromCharacterCode", "[", 
  RowBox[{"{", 
   RowBox[{"87", ",", "88", ",", "89", ",", "90"}], "}"}], "]"}]], "Input",
 CellLabel->"In[8]:="],

Cell[BoxData["\<\"WXYZ\"\>"], "Output",
 CellChangeTimes->{3.465771611115*^9, 3.4658598441616*^9, 3.46859774109416*^9,
   3.468597864494225*^9, 3.470067177458447*^9},
 CellLabel->"Out[8]="]
}, Open  ]],

Cell["The mapping may not be one-to-one for certain encodings.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FromCharacterCode", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"206", ",", "177"}], "}"}], ",", " ", "\"\<UTF8\>\""}], 
  "]"}]], "Input",
 CellLabel->"In[9]:="],

Cell[BoxData["\<\"\[Alpha]\"\>"], "Output",
 CellChangeTimes->{3.4657716112553997`*^9, 3.4658598441836*^9, 
  3.4685977411430397`*^9, 3.468597864544539*^9, 3.470067177501834*^9},
 CellLabel->"Out[9]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["String and Regular Expressions", "Heading2"],

Cell[TextData[{
 StyleBox["A great deal of Mathematica\[CloseCurlyQuote]s prowess in text \
processing comes ",
  FontTracking->"Plain"],
 StyleBox["from its rich support",
  FontTracking->"SemiCondensed"],
 StyleBox[" for patt",
  FontTracking->"Plain"],
 "ern matching. There are two basic classes of string patterns: string \
expressions and regular expressions. Introduced in version 5.1, each has a \
similar expressive power. The advantage of ",
 StyleBox["StringExpression", "Literal"],
 " is that it is less cryptic because it ",
 StyleBox["uses more words than symbols to express patterns. The advantage of \
",
  FontTracking->"SemiCondensed"],
 StyleBox["RegularExpression", "Literal",
  FontTracking->"SemiCondensed"],
 " is that it is more standa",
 StyleBox["rdized with other languages such as Perl, Ruby, ",
  FontTracking->"Plain"],
 StyleBox["Java, and so on. Non-Mathematica",
  FontTracking->"SemiCondensed"],
 StyleBox[" progr",
  FontTracking->"Plain"],
 "ammers, especially those with a background in Unix, are more likely to \
understand regular expressions, although these expressions are cryptic to the \
uninitiated. You should become familiar with both if you plan to do much \
string manipulation. If you program frequently in languages outside of ",
 StyleBox["Mathematica, master the regular expression syntax. If you work \
strictly in Mathematica,",
  FontTracking->"SemiCondensed"],
 " choose the one that most appeals to you. If you learn the string \
expression syntax, you will have a leg up on learning Mathematica\
\[CloseCurlyQuote]s more general pattern-matching syntax, which is used in \
many contexts outside text processing. You can also mix string expressions \
and regular expressions into compound patterns."
}], "Text",
 CellChangeTimes->{{3.470160424314131*^9, 3.470160461606842*^9}, {
   3.470160630018032*^9, 3.470160631645801*^9}, {3.470160666861655*^9, 
   3.47016075622622*^9}, {3.470160813957899*^9, 3.470160866334325*^9}, {
   3.470160915124004*^9, 3.470161107542222*^9}, {3.470516688489731*^9, 
   3.470516699297923*^9}, 3.47162396894215*^9}],

Cell[CellGroupData[{

Cell["String expressions     ", "Heading3",
 CellChangeTimes->{3.475586842325119*^9}],

Cell[TextData[{
 StyleBox["StringExpressions", "Literal"],
 StyleBox[" are mostly written",
  FontTracking->"SemiCondensed"],
 " using the infix operator ",
 StyleBox["~~", "Literal"],
 ", which is a syntactic shortcut for the ",
 StyleBox["StringExpression[", "Literal"],
 StyleBox["]", "Literal",
  FontTracking->"SemiCondensed"],
 StyleBox[" function. ",
  FontTracking->"SemiCondensed"],
 StyleBox["S", "Literal",
  FontTracking->"SemiCondensed"],
 StyleBox["tringExpression", "Literal"],
 " uses Mathematica\[CloseCurlyQuote]s blanks notation (e.g., ",
 StyleBox["_", "Literal"],
 ", ",
 StyleBox["__", "Literal"],
 ", and ",
 StyleBox["___", "Literal"],
 ") to represent wild cards. See ",
 ButtonBox["Chapter 4",
  BaseStyle->"Hyperlink",
  ButtonData->{"Patterns.nb", "IntroductionCh4"}],
 " for more on blanks."
}], "Text",
 CellChangeTimes->{
  3.4657711127686*^9, {3.4701611506995983`*^9, 3.470161175167523*^9}, {
   3.470161209870078*^9, 3.47016123464215*^9}, {3.470161266202001*^9, 
   3.4701612824484587`*^9}, 3.4705167039731293`*^9}],

Cell[TextData[{
 "Match ",
 StyleBox["\"xy\"", "Literal"],
 " followed by any character."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\"\<xy\>\"", " ", "~~", " ", "_"}], ";"}]], "Input",
 CellLabel->"In[10]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<xyz\>\"", " ", ",", " ", 
   RowBox[{"\"\<xy\>\"", " ", "~~", " ", "_"}]}], "]"}]], "Input",
 CellLabel->"In[11]:="],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.4657716114114*^9, 3.4658598442106*^9, 
  3.468597741209527*^9, 3.468597864626727*^9, 3.470067177584592*^9},
 CellLabel->"Out[11]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<xyzz\>\"", " ", ",", " ", 
   RowBox[{"\"\<xy\>\"", " ", "~~", " ", "_"}]}], "]"}]], "Input",
 CellLabel->"In[12]:="],

Cell[BoxData["False"], "Output",
 CellChangeTimes->{3.4657716114582*^9, 3.4658598442316*^9, 
  3.468597741243548*^9, 3.4685978646607533`*^9, 3.470067177619108*^9},
 CellLabel->"Out[12]="]
}, Open  ]],

Cell[TextData[{
 "Match ",
 StyleBox["\"xy\"", "Literal"],
 " followed by one or more characters."
}], "Text", "ProductionPageBreak"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\"\<xy\>\"", " ", "~~", " ", "__"}], ";"}]], "Input",
 CellLabel->"In[13]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<xyzz\>\"", " ", ",", " ", 
   RowBox[{"\"\<xy\>\"", " ", "~~", " ", "__"}]}], "]"}]], "Input",
 CellLabel->"In[14]:="],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.4657716115986*^9, 3.4658598442566*^9, 
  3.468597741309764*^9, 3.46859786472647*^9, 3.4700671777008*^9},
 CellLabel->"Out[14]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<xy\>\"", " ", ",", " ", 
   RowBox[{"\"\<xy\>\"", " ", "~~", " ", "__"}]}], "]"}]], "Input",
 CellLabel->"In[15]:="],

Cell[BoxData["False"], "Output",
 CellChangeTimes->{3.4657716116454*^9, 3.4658598442785997`*^9, 
  3.468597741343399*^9, 3.4685978647655478`*^9, 3.47006717774658*^9},
 CellLabel->"Out[15]="]
}, Open  ]],

Cell[TextData[{
 "Match ",
 StyleBox["\"xy\"", "Literal"],
 " followed by zero or more characters."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\"\<xy\>\"", " ", "~~", " ", "___"}], ";"}]], "Input",
 CellLabel->"In[16]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<xyz\>\"", " ", ",", " ", 
   RowBox[{"\"\<xy\>\"", " ", "~~", " ", "___"}]}], "]"}]], "Input",
 CellLabel->"In[17]:="],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.4657716117078*^9, 3.4658598443046*^9, 
  3.468597741426845*^9, 3.468597864827278*^9, 3.470067177818186*^9},
 CellLabel->"Out[17]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<xy\>\"", " ", ",", " ", 
   RowBox[{"\"\<xy\>\"", " ", "~~", " ", "___"}]}], "]"}]], "Input",
 CellLabel->"In[18]:="],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.465771611817*^9, 3.4658598443266*^9, 
  3.468597741463381*^9, 3.468597864864677*^9, 3.470067177933321*^9},
 CellLabel->"Out[18]="]
}, Open  ]],

Cell["\<\
Patterns can be associated with variables so that the matching portion can be \
referred to in a subsequent expression. For example, the following pattern \
will match if the string begins and ends with the same sequence of characters.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<xyx\>\"", " ", ",", " ", 
   RowBox[{"x__", "~~", " ", "___", " ", "~~", "x__"}]}], "]"}]], "Input",
 CellLabel->"In[19]:="],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.4657716119574003`*^9, 3.4658598443486*^9, 
  3.4685977415104647`*^9, 3.468597864898919*^9, 3.4700671779799967`*^9},
 CellLabel->"Out[19]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<Hello. I said, hello\>\"", " ", ",", " ", 
   RowBox[{"x__", "~~", " ", "___", " ", "~~", "x__"}], ",", " ", 
   RowBox[{"IgnoreCase", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellLabel->"In[20]:="],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.4657716120666*^9, 3.4658598443696003`*^9, 
  3.468597741564735*^9, 3.46859786494489*^9, 3.4700671780196333`*^9},
 CellLabel->"Out[20]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<123ABC323\>\"", ",", " ", 
   RowBox[{
    RowBox[{"x", ":", "NumberString"}], " ", "~~", " ", "___", " ", "~~", " ",
     "x_"}]}], "]"}]], "Input",
 CellChangeTimes->{3.4755872223350563`*^9},
 CellLabel->"In[21]:="],

Cell[BoxData["False"], "Output",
 CellChangeTimes->{3.4657716126438*^9, 3.4658598443906*^9, 
  3.4685977416292343`*^9, 3.468597864981778*^9, 3.4700671784221354`*^9},
 CellLabel->"Out[21]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<123ABC123\>\"", ",", " ", 
   RowBox[{
    RowBox[{"x", ":", "NumberString"}], " ", "~~", " ", "___", " ", "~~", " ",
     "x_"}]}], "]"}]], "Input",
 CellChangeTimes->{3.475587227676667*^9},
 CellLabel->"In[22]:="],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.4657716128310003`*^9, 3.4658598444046*^9, 
  3.4685977416814137`*^9, 3.468597865012238*^9, 3.470067178467586*^9},
 CellLabel->"Out[22]="]
}, Open  ]],

Cell[TextData[{
 ButtonBox["Table 5-1",
  BaseStyle->"Hyperlink",
  ButtonData->"tStringPatterns"],
 " shows some of the common raw ingredients for string expressions. If you \
have already read ",
 ButtonBox["Chapter 4",
  BaseStyle->"Hyperlink",
  ButtonData->{"Patterns.nb, chPatterns"}],
 " on pattern matching, you can see that all the same constructs are \
available for strings. The full set of string expression primitives can be \
found in ",
 StyleBox[ButtonBox["tutorial/WorkingWithStringPattern",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://reference.wolfram.com/mathematica/tutorial/\
WorkingWithStringPatterns.html"], None},
  ButtonNote->
   "http://reference.wolfram.com/mathematica/tutorial/\
WorkingWithStringPatterns.html"], "URL"],
 StyleBox["s", "URL"],
 "."
}], "Text",
 CellChangeTimes->{
  3.470161397729347*^9, {3.47016143261559*^9, 3.4701614485030403`*^9}, {
   3.470517323995583*^9, 3.4705173240035973`*^9}, 3.470575663383222*^9, {
   3.470575711653949*^9, 3.4705757116904497`*^9}, {3.471624228887227*^9, 
   3.471624238304882*^9}, 3.475587846989828*^9}],

Cell["Common string patterns", "TableTitle", "TopOfPage",
 CellChangeTimes->{{3.475587584688957*^9, 3.47558758800677*^9}},
 CellTags->"tStringPatterns"],

Cell[BoxData[
 TagBox[GridBox[{
    {"", "", "Pattern", "Description", "", ""},
    {"", "", "", "", "", ""},
    {"", "", 
     RowBox[{"\"\<\\\"string\\\"\>\"", "\[IndentingNewLine]", "\"\<_\>\""}], 
     RowBox[{
     "\"\<a literal string of characters\>\"", "\[IndentingNewLine]", 
      "\"\<any single character\>\""}], "", ""},
    {"", "", "\"\<__\>\"", "\"\<any substring of one or more characters\>\"", 
     "", ""},
    {"", "", "\"\<___\>\"", 
     "\"\<any substring of zero or more characters\>\"", "", ""},
    {"", "", "\"\<x_,x__,x___\>\"", "\"\<substrings given the name x\>\"", "",
      ""},
    {"", "", "\"\<x:pattern\>\"", "\"\<pattern given the name x\>\"", "", 
     ""},
    {"", "", "\"\<pattern..\>\"", 
     "\"\<pattern repeated one or more times\>\"", "", ""},
    {"", "", "\"\<pattern...\>\"", 
     "\"\<pattern repeated zero or more times\>\"", "", ""},
    {"", "", "\"\<patt1|patt2|...\>\"", 
     "\"\<a pattern matching at least one of the patt-i\>\"", "", ""},
    {"", "", "\"\<patt/;cond\>\"", 
     "\"\<a pattern for which cond evaluates to True\>\"", "", ""},
    {"", "", "\"\<pattern?test\>\"", 
     "\"\<a pattern for which test yields True for each character\>\"", "", 
     ""},
    {"", "", "\"\<Except[pattern]\>\"", 
     "\"\<matches anything except pattern\>\"", "", ""},
    {"", "", "\"\<Whitespace\>\"", 
     "\"\<a sequence of whitespace characters\>\"", "", ""},
    {"", "", "\"\<NumberString\>\"", "\"\<the characters of a number\>\"", "",
      ""},
    {"", "", "\"\<DatePattern[spec]\>\"", "\"\<the characters of a date\>\"", 
     "", ""},
    {"", "", "\"\<charobj\>\"", 
     "\"\<an object representing a character class (see below)\>\"", "", ""}
   }],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Table",
 Evaluatable->False,
 CellChangeTimes->{{3.475587311388282*^9, 3.475587377451276*^9}, {
  3.476615721429531*^9, 3.476615731114831*^9}},
 GridBoxOptions->{
 GridBoxSpacings->{
  "Columns" -> {Automatic, Automatic, {2.1}, Automatic, Automatic}}},
 CellLabel->""],

Cell[TextData[{
 StyleBox[ButtonBox["Table 5-2",
  BaseStyle->"Hyperlink",
  ButtonData->"tRegExPatterns"],
  FontTracking->"SemiCondensed"],
 StyleBox[" shows some of the common raw ingredients for regular expressions.",
  FontTracking->"SemiCondensed"],
 StyleBox[" The full set of regular expression primitives can be found in ",
  FontTracking->"Plain"],
 StyleBox[ButtonBox["tutorial/WorkingWithStringPattern",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://reference.wolfram.com/mathematica/tutorial/\
WorkingWithStringPatterns.html"], None},
  ButtonNote->
   "http://reference.wolfram.com/mathematica/tutorial/\
WorkingWithStringPatterns.html"], "URL",
  FontTracking->"Plain"],
 StyleBox["s", "URL",
  FontTracking->"Plain"],
 StyleBox[".",
  FontTracking->"Plain"],
 " Here ",
 StyleBox["c", "Literal"],
 " or ",
 StyleBox["c", "Literal"],
 StyleBox["n", "LiteralItalic"],
 ",",
 " where ",
 StyleBox["n", "LiteralItalic"],
 " is a number, is a placeholder for an arbitrary character, and ",
 StyleBox["p", "Literal"],
 StyleBox["n", "LiteralItalic"],
 " is a placeholder for an arbitrary regular expression."
}], "Text",
 CellMargins->{{0, 0}, {2, 23}},
 CellChangeTimes->{{3.47048324151019*^9, 3.470483250924515*^9}, {
   3.470575635222534*^9, 3.470575635258768*^9}, 3.4705780250698557`*^9, {
   3.471624494412591*^9, 3.471624530661346*^9}}]
}, Open  ]]
}, Open  ]],

Cell["Common regular expressions", "TableTitle",
 CellMargins->{{0, 0}, {0, 4}},
 CellChangeTimes->{{3.4755875919272747`*^9, 3.475587594279258*^9}},
 CellTags->"tRegExPatterns"],

Cell[BoxData[
 TagBox[GridBox[{
    {"", "", 
     RowBox[{"Regular", " ", "expression"}], "Description", "", ""},
    {"", "", "", "", "", ""},
    {"", "", 
     RowBox[{"\"\<[c1c2c3]\>\"", "\[IndentingNewLine]", "\"\<[c1-c2]\>\""}], 
     RowBox[{
     "\"\<Matches any of the characters c1, c2, or c3. For \
example,[AEIOUaeiou] matches vowels.\>\"", "\[IndentingNewLine]", 
      "\"\<Matches characters c1 through c2. For example, [a-z] matches all \
lowercase letters.\>\""}], "", ""},
    {"", "", "\"\<[^c1c2c3]\>\"", 
     "\"\<Matches any characters EXCEPT c1, c2, c3. For example,[^AEIOUaeiou] \
matches nonvowels.\>\"", "", ""},
    {"", "", "\"\<c*\>\"", 
     "\"\<Zero or more occurrences of character c. Greedy version.\>\"", "", 
     ""},
    {"", "", "\"\<c+\>\"", 
     "\"\<One or more occurrences of character c. Greedy version.\>\"", "", 
     ""},
    {"", "", "\"\<c?\>\"", 
     "\"\<The character c or nothing (i.e., zero or one occurrences). Greedy \
version.\>\"", "", ""},
    {"", "", "\"\<c*?\>\"", "\"\<Lazy version of c*.\>\"", "", ""},
    {"", "", "\"\<c+?\>\"", "\"\<Lazy version of c+.\>\"", "", ""},
    {"", "", "\"\<c??\>\"", "\"\<Lazy version of c?.\>\"", "", ""},
    {"", "", "\"\<p1|p2|...|pN\>\"", "\"\<Matches p1 or p2 or ... pN.\>\"", 
     "", ""},
    {"", "", "\"\<p1p2...pN\>\"", 
     "\"\<Matches p1, followed by p2, followed by ... pN.\>\"", "", ""},
    {"", "", "\"\<^p1\>\"", 
     "\"\<Matches p1 only at the start of the string.\>\"", "", ""},
    {"", "", "\"\<p1$\>\"", 
     "\"\<Matches p1 only at the end of the string.\>\"", "", ""},
    {"", "", "\"\<^p1$\>\"", 
     "\"\<Matches only if p1 matches the entire string.\>\"", "", ""},
    {"", "", "\"\<\\\\d\>\"", "\"\<Any digit 0-9\>\"", "", ""},
    {"", "", "\"\<\\\\s\>\"", "\"\<Whitespace\>\"", "", ""}
   }],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Table",
 Evaluatable->False,
 CellChangeTimes->{{3.4704832893704033`*^9, 3.470483427306027*^9}, 
   3.4704834615102777`*^9, {3.475587521962775*^9, 3.475587551847907*^9}, {
   3.475587600024575*^9, 3.475587611741624*^9}, {3.476615747709415*^9, 
   3.4766157609558563`*^9}},
 GridBoxOptions->{
 GridBoxSpacings->{
  "Columns" -> {
    Automatic, Automatic, {1.0999999999999992`}, Automatic, Automatic}}},
 CellLabel->""],

Cell[CellGroupData[{

Cell["See Also", "Heading2",
 CellMargins->{{0, 36}, {0, 5}}],

Cell[TextData[{
 StyleBox["The definitive reference on regular expressions is ",
  FontTracking->"Plain"],
 StyleBox["Mastering Regular Expressions, ", "CitationTitle",
  FontTracking->"Plain"],
 StyleBox["Second Edition, by Jeffrey E. F. Friedl \
(O\[CloseCurlyQuote]Reilly). If you plan to do anything nontrivial using \
regular expression matching, you will save yourself hours of frustration by \
consulting this book.",
  FontTracking->"Plain"]
}], "Text",
 CellChangeTimes->{
  3.4657711515657997`*^9, {3.4704839195812187`*^9, 3.470483995771308*^9}, {
   3.470484208458674*^9, 3.470484256867345*^9}, {3.470484351663417*^9, 
   3.470484391537698*^9}, {3.4705174841168947`*^9, 3.4705174841237373`*^9}, {
   3.470517535045076*^9, 3.470517535053069*^9}, {3.470664969951625*^9, 
   3.470664972050399*^9}, {3.475588283394569*^9, 3.475588284943568*^9}}]
}, Open  ]]
}, Open  ]],

Cell[TextData[{
 StyleBox["An excellent tutorial on working with string patterns in \
Mathematica can be ",
  FontTracking->"Extended"],
 StyleBox["found in the documentation under ",
  FontTracking->"Plain"],
 StyleBox[ButtonBox["tutorial/WorkingWithStringPatterns",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://reference.wolfram.com/mathematica/tutorial/\
WorkingWithStringPatterns.html"], None},
  ButtonNote->
   "http://reference.wolfram.com/mathematica/tutorial/\
WorkingWithStringPatterns.html"], "URL",
  FontTracking->"Plain"],
 StyleBox[" or online",
  FontTracking->"Plain"],
 StyleBox[" at ",
  FontTracking->"Extended"],
 StyleBox[ButtonBox["http",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://bit.ly/yGbND"], None},
  ButtonNote->"http://bit.ly/yGbND"], "URL",
  FontTracking->"Extended"],
 StyleBox[ButtonBox["://bit.ly/yGbND",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://bit.ly/yGbND"], None},
  ButtonNote->"http://bit.ly/yGbND"], "URL"],
 ". Besides being a good all-around tutorial, it has a section specifically \
targeting Perl programmers, which is helpful for those who already have \
experience with string manipulation in Perl."
}], "Text"],

Cell[TextData[{
 
 CounterBox["ChapterLabel"],
 ".",
 
 CounterBox["Heading1"],
 "\[ThickSpace]\[ThickSpace]",
 "Comparing Strings"
}], "Heading1", "ProductionPageBreak",
 CellTags->"ComapringStrings"],

Cell[CellGroupData[{

Cell["Problem", "Heading2WithHeadAbove"],

Cell[TextData[{
 "You want to compare strings but ",
 StyleBox["Less", "Literal"],
 ", ",
 StyleBox["LessEqual", "Literal"],
 ", ",
 StyleBox["Greater", "Literal"],
 ", and ",
 StyleBox["GreaterEqual", "Literal"],
 " do not work."
}], "Text"]
}, Open  ]],

Cell["Solution", "Heading2"],

Cell[TextData[{
 "Use ",
 StyleBox["Order[e1,e2]", "Literal"],
 ", which returns 1 if e1 is before e2, \[Dash]1 if e1 is after e2, and 0 if \
they are equal."
}], "Text",
 CellChangeTimes->{{3.470493490496502*^9, 3.470493534723013*^9}, 
   3.47162467432055*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Order", "[", 
  RowBox[{"\"\<rat\>\"", ",", "\"\<rate\>\""}], "]"}]], "Input",
 CellLabel->"In[23]:="],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.4657716130025997`*^9, 3.4658598445126*^9, 
  3.468597741995742*^9, 3.468597865045491*^9, 3.470067178513153*^9},
 CellLabel->"Out[23]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Order", "[", 
  RowBox[{"\"\<rat\>\"", ",", "\"\<cat\>\""}], "]"}]], "Input",
 CellLabel->"In[24]:="],

Cell[BoxData[
 RowBox[{"-", "1"}]], "Output",
 CellChangeTimes->{3.4657716131118*^9, 3.4658598445336*^9, 
  3.468597742033165*^9, 3.4685978650820217`*^9, 3.470067178553547*^9},
 CellLabel->"Out[24]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Discussion", "Heading2"],

Cell[TextData[{
 "Most users of Mathematica will not find themselves doing direct string \
comparison since functions like ",
 StyleBox["Sort", "Literal"],
 " and ",
 StyleBox["Ordering", "Literal"],
 " do the right thing. However, if you find yourself needing to use the more \
natural comparison operators with strings, you can do the following:"
}], "Text",
 CellChangeTimes->{
  3.470493557231374*^9, {3.4704937632567253`*^9, 3.470493774441434*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Unprotect", "[", 
   RowBox[{
   "Less", ",", " ", "LessEqual", ",", "Greater", ",", "GreaterEqual"}], 
   "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Less", "[", 
    RowBox[{"s1_String", ",", "s2_String"}], "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"Order", "[", 
     RowBox[{"s1", ",", "s2"}], "]"}], " ", ">", " ", "0"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"LessEqual", "[", 
    RowBox[{"s1_String", ",", "s2_String"}], "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"Order", "[", 
     RowBox[{"s1", ",", "s2"}], "]"}], " ", ">", " ", 
    RowBox[{"-", "1"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Greater", "[", 
    RowBox[{"s1_String", ",", "s2_String"}], "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"Order", "[", 
     RowBox[{"s1", ",", "s2"}], "]"}], " ", "<", " ", "0"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"GreaterEqual", "[", 
    RowBox[{"s1_String", ",", "s2_String"}], "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"Order", "[", 
     RowBox[{"s1", ",", "s2"}], "]"}], " ", "<", " ", "1"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Protect", "[", 
   RowBox[{
   "Less", ",", " ", "LessEqual", ",", "Greater", ",", "GreaterEqual"}], 
   "]"}], ";"}], "\[IndentingNewLine]"}], "Input",
 CellLabel->"In[25]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\"\<rat\>\"", " ", "<", " ", "\"\<cat\>\""}]], "Input",
 CellLabel->"In[31]:="],

Cell[BoxData["False"], "Output",
 CellChangeTimes->{3.4657716133302*^9, 3.4658598445626*^9, 
  3.468597742111717*^9, 3.468597865193103*^9, 3.470067178619945*^9},
 CellLabel->"Out[31]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\"\<cat\>\"", " ", "<", " ", "\"\<rat\>\""}]], "Input",
 CellLabel->"In[32]:="],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.4657716134393997`*^9, 3.4658598445846*^9, 
  3.4685977421686277`*^9, 3.4685978652318287`*^9, 3.4700671786700974`*^9},
 CellLabel->"Out[32]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\"\<cat\>\"", " ", "<=", " ", "\"\<cat\>\""}]], "Input",
 CellLabel->"In[33]:="],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.4657716135486*^9, 3.4658598446066*^9, 
  3.4685977422117157`*^9, 3.468597865261297*^9, 3.4700671787035427`*^9},
 CellLabel->"Out[33]="]
}, Open  ]]
}, Open  ]],

Cell[TextData[{
 
 CounterBox["ChapterLabel"],
 ".",
 
 CounterBox["Heading1"],
 "\[ThickSpace]\[ThickSpace]Removing and Replacing Characters ",
 " ",
 "from Strings"
}], "Heading1", "ProductionPageBreak",
 CellChangeTimes->{3.470576388499741*^9, 3.485522430229926*^9},
 CellTags->"Removing and Replacing Characters from Strings"],

Cell[CellGroupData[{

Cell["Problem", "Heading2WithHeadAbove"],

Cell[TextData[{
 StyleBox["You want to strip certain characters (e.g., whitespace) or \
characters at certain positions ",
  FontTracking->"SemiCondensed"],
 StyleBox["from a string. You may also want to replace these characters with \
other characters.",
  FontTracking->"Plain"]
}], "Text",
 CellChangeTimes->{3.4657711595685997`*^9, 3.470493893525062*^9}]
}, Open  ]],

Cell["Solution", "Heading2",
 CellMargins->{{0, 36}, {0, 9}}],

Cell[CellGroupData[{

Cell["Using patterns", "Heading3WithHeadAbove",
 CellChangeTimes->{3.475588342911291*^9}],

Cell[TextData[{
 StyleBox["StringReplace[]", "Literal",
  FontTracking->"Plain"],
 StyleBox[" is an extremely versatile function that ",
  FontTracking->"SemiCondensed"],
 StyleBox["solves most character-oriented",
  FontTracking->"Plain"],
 StyleBox[" ",
  FontTracking->"SemiCondensed"],
 StyleBox["stripping and replacing operations.",
  FontTracking->"Plain"],
 StyleBox[" It supports a very ",
  FontTracking->"SemiCondensed"],
 StyleBox["general set of string-substitution",
  FontTracking->"Plain"],
 StyleBox[" ",
  FontTracking->"SemiCondensed"],
 StyleBox["rules, including regular expressions and Mathematica-specific \
string patterns.",
  FontTracking->"Plain"]
}], "Text",
 CellChangeTimes->{{3.470493902280719*^9, 3.470493936232353*^9}, 
   3.4705176395963297`*^9}],

Cell["Strip all spaces.", "Text",
 CellMargins->{{0, 0}, {2, 3}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
  "myString", " ", "=", " ", 
   "\"\< The quick brown  fox   jumped over the lazy programmer   \>\""}], 
  ";"}]], "Input",
 CellMargins->{{53, 0}, {0, 2}},
 CellLabel->"In[34]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringReplace", "[", 
  RowBox[{"myString", ",", " ", 
   RowBox[{"\"\< \>\"", " ", "\[Rule]", " ", "\"\<\>\""}]}], "]"}]], "Input",
 CellLabel->"In[35]:="],

Cell[BoxData["\<\"Thequickbrownfoxjumpedoverthelazyprogrammer\"\>"], "Output",
 CellChangeTimes->{3.4657716138294*^9, 3.4658598446336*^9, 
  3.468597742296372*^9, 3.468597865312623*^9, 3.470067178769808*^9},
 CellLabel->"Out[35]="]
}, Open  ]],

Cell["Strip leading and trailing whitespace.", "Text",
 CellMargins->{{0, 0}, {2, 3}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"StringReplace", "[", 
   RowBox[{"myString", ",", " ", 
    RowBox[{
     RowBox[{"RegularExpression", "[", "\"\<^\\\\s+|\\\\s+$\>\"", "]"}], 
     "\[Rule]", "\"\<\>\""}]}], "]"}], "//", "InputForm"}]], "Input",
 CellMargins->{{53, 0}, {0, 2}},
 CellLabel->"In[36]:="],

Cell[TextData[StyleBox["\"The quick brown  fox   jumped over the lazy \
programmer\"",
 FontFamily->"WriTheSansMonoCond BoldItalic"]], "Output",
 CellChangeTimes->{3.4657716139698*^9, 3.4658598446566*^9, 
  3.468597742345622*^9, 3.468597865367098*^9, 3.470067178807686*^9},
 CellLabel->"Out[36]//InputForm="]
}, Open  ]],

Cell["\<\
Normalize whitespace: strip leading, trailing, and multiple internal \
whitespace.\
\>", "Text",
 CellMargins->{{0, 0}, {2, 1}},
 CellChangeTimes->{{3.470494031139451*^9, 3.470494040219131*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"StringReplace", "[", 
   RowBox[{"myString", ",", "  ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"RegularExpression", "[", "\"\<^\\\\s+|\\\\s+$\>\"", "]"}], 
       "\[Rule]", " ", "\"\<\>\""}], ",", " ", 
      RowBox[{
       RowBox[{"RegularExpression", "[", "\"\<\\\\s\\\\s+\>\"", "]"}], 
       "\[Rule]", " ", "\"\< \>\""}]}], "}"}]}], "]"}], "//", 
  "InputForm"}]], "Input",
 CellMargins->{{53, 0}, {0, 2}},
 CellLabel->"In[37]:="],

Cell[TextData[StyleBox["\"The quick brown fox jumped over the lazy programmer\
\"",
 FontFamily->"WriTheSansMonoCond BoldItalic"]], "Output",
 CellChangeTimes->{3.4657716141102*^9, 3.4658598446786003`*^9, 
  3.468597742399407*^9, 3.4685978654130573`*^9, 3.470067178841951*^9},
 CellLabel->"Out[37]//InputForm="]
}, Open  ]],

Cell["Literal string substitution.", "Text",
 CellMargins->{{0, 0}, {2, 1}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"StringReplace", "[", 
   RowBox[{"myString", ",", " ", 
    RowBox[{"\"\<the\>\"", " ", "\[Rule]", " ", "\"\<a\>\""}]}], "]"}], "//", 
  "InputForm"}]], "Input",
 CellMargins->{{53, 0}, {0, 2}},
 CellLabel->"In[38]:="],

Cell[TextData[StyleBox["\" The quick brown  fox   jumped over a lazy \
programmer   \"",
 FontFamily->"WriTheSansMonoCond BoldItalic"]], "Output",
 CellChangeTimes->{3.4657716142506*^9, 3.4658598446996*^9, 
  3.468597742445953*^9, 3.468597865449356*^9, 3.4700671789537992`*^9},
 CellLabel->"Out[38]//InputForm="]
}, Open  ]],

Cell["Ignore case while matching.", "Text",
 CellMargins->{{0, 0}, {2, 1}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"StringReplace", "[", 
   RowBox[{"myString", ",", " ", 
    RowBox[{"\"\<the\>\"", " ", "\[Rule]", " ", "\"\<a\>\""}], ",", " ", 
    RowBox[{"IgnoreCase", "\[Rule]", "True"}]}], "]"}], "//", 
  "InputForm"}]], "Input",
 CellMargins->{{53, 0}, {0, 2}},
 CellLabel->"In[39]:="],

Cell[TextData[StyleBox["\" a quick brown  fox   jumped over a lazy programmer \
   \"",
 FontFamily->"WriTheSansMonoCond BoldItalic"]], "Output",
 CellChangeTimes->{3.465771614391*^9, 3.4658598447216*^9, 
  3.468597742502659*^9, 3.468597865482511*^9, 3.470067179003845*^9},
 CellLabel->"Out[39]//InputForm="]
}, Open  ]],

Cell["\<\
Use Mathematica-specific patterns instead of regular expressions.\
\>", "Text",
 CellMargins->{{0, 0}, {2, -1}},
 CellChangeTimes->{{3.4704940894139357`*^9, 3.470494094449724*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringReplace", "[", 
  RowBox[{"myString", ",", " ", 
   RowBox[{
    RowBox[{"\"\<ox\>\"", "~~", "Whitespace"}], "\[Rule]", "\"\<ox \>\""}]}], 
  "]"}]], "Input",
 CellMargins->{{53, 0}, {0, 2}},
 CellLabel->"In[40]:="],

Cell[BoxData["\<\" The quick brown  fox jumped over the lazy programmer   \
\"\>"], "Output",
 CellChangeTimes->{3.4657716145313997`*^9, 3.4658598447436*^9, 
  3.468597742545907*^9, 3.468597865533609*^9, 3.470067179041217*^9},
 CellLabel->"Out[40]="]
}, Open  ]]
}, Open  ]],

Cell["Using positions", "Heading3", "ProductionPageBreak",
 CellChangeTimes->{{3.475588450249978*^9, 3.475588453363584*^9}}],

Cell[TextData[{
 "Sometimes you know exactly where the characters are that you want to \
remove. In that case, ",
 StyleBox["StringDrop[]", "Literal"],
 " is a lot more efficient. ",
 StyleBox["StringDrop[]", "Literal"],
 " takes the string and a second argument, which can be an offset from the \
front, an offset from the end, specific positions, or a range of positions."
}], "Text",
 CellChangeTimes->{{3.470494120783388*^9, 3.470494157332897*^9}}],

Cell["Consider:", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myString", " ", "=", " ", "\"\<abcdefghijklmnop\>\""}], " ", 
  ";"}]], "Input",
 CellLabel->"In[41]:="],

Cell["Here you drop the first three characters. ", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringDrop", "[", 
  RowBox[{"myString", ",", " ", "3"}], "]"}]], "Input",
 CellLabel->"In[42]:="],

Cell[BoxData["\<\"defghijklmnop\"\>"], "Output",
 CellChangeTimes->{3.4657716147654*^9, 3.4658598447706003`*^9, 
  3.46859774262838*^9, 3.468597865595663*^9, 3.470067179103627*^9},
 CellLabel->"Out[42]="]
}, Open  ]],

Cell["Alternatively, you drop the last three characters, like so.", "Text",
 CellChangeTimes->{{3.470494182016325*^9, 3.470494187406994*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringDrop", "[", 
  RowBox[{"myString", ",", " ", 
   RowBox[{"-", "3"}]}], "]"}]], "Input",
 CellLabel->"In[43]:="],

Cell[BoxData["\<\"abcdefghijklm\"\>"], "Output",
 CellChangeTimes->{3.4657716149681997`*^9, 3.4658598447916*^9, 
  3.468597742683465*^9, 3.4685978656291*^9, 3.470067179155072*^9},
 CellLabel->"Out[43]="]
}, Open  ]],

Cell["Drop only the third character, like this.", "Text",
 CellChangeTimes->{3.470494198721588*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringDrop", "[", 
  RowBox[{"myString", ",", " ", 
   RowBox[{"{", "3", "}"}]}], "]"}]], "Input",
 CellLabel->"In[44]:="],

Cell[BoxData["\<\"abdefghijklmnop\"\>"], "Output",
 CellChangeTimes->{3.4657716151085997`*^9, 3.4658598448126*^9, 
  3.468597742729006*^9, 3.468597865666214*^9, 3.4700671791919127`*^9},
 CellLabel->"Out[44]="]
}, Open  ]],

Cell[TextData[{
 "And drop the third through fifth (",
 StyleBox["\"cde\"", "Literal"],
 "), using a range list."
}], "Text",
 CellChangeTimes->{{3.470494207754469*^9, 3.4704942255553503`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringDrop", "[", 
  RowBox[{"myString", ",", " ", 
   RowBox[{"{", 
    RowBox[{"3", ",", "5"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[45]:="],

Cell[BoxData["\<\"abfghijklmnop\"\>"], "Output",
 CellChangeTimes->{3.465771615249*^9, 3.4658598448346*^9, 
  3.468597742783856*^9, 3.468597865699752*^9, 3.470067179237932*^9},
 CellLabel->"Out[45]="],

Cell[TextData[{
 "The step size in the range can even be",
 StyleBox[" greater than one by specifying it",
  FontTracking->"Plain"],
 StyleBox[" as the third el",
  FontTracking->"Extended"],
 "ement. Here you specify a step size of two to remove every other character. \
The ",
 StyleBox["-1", "Literal"],
 " upper limit is a convenient way to specify the end of the string without \
having to know its length."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringDrop", "[", 
  RowBox[{"myString", ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}], ",", "2"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[46]:="],

Cell[BoxData["\<\"bdfhjlnp\"\>"], "Output",
 CellChangeTimes->{3.4657716153894*^9, 3.4658598448596*^9, 
  3.468597742878985*^9, 3.4685978658632*^9, 3.4700671793259487`*^9},
 CellLabel->"Out[46]="]
}, Open  ]]
}, Open  ]],

Cell["You can also act on several strings at once.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"otherString", " ", "=", " ", "\"\<1234567890\>\""}], 
  ";"}]], "Input",
 CellLabel->"In[47]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringDrop", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"myString", ",", " ", "otherString"}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"3", ",", "5"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[48]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"abfghijklmnop\"\>", ",", "\<\"1267890\"\>"}], "}"}]], "Output",\

 CellChangeTimes->{3.4657716155922003`*^9, 3.4658598448896*^9, 
  3.468597743028702*^9, 3.46859786592939*^9, 3.470067179385673*^9},
 CellLabel->"Out[48]="]
}, Open  ]],

Cell[TextData[{
 "The positional form for replacement is called ",
 StyleBox["StringReplacePart[]", "Literal"],
 ", and it works using similar conventions for specifying positions. The \
difference is that you must",
 " ",
 "always provide a contiguous range or a list of such ranges."
}], "Text",
 CellChangeTimes->{
  3.470494255586812*^9, {3.470517739940673*^9, 3.470517758423729*^9}, {
   3.485522430960802*^9, 3.48552243169655*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringReplacePart", "[", 
  RowBox[{"myString", ",", " ", "\"\<ZZZ\>\"", ",", " ", 
   RowBox[{"{", 
    RowBox[{"3", ",", "5"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[49]:="],

Cell[BoxData["\<\"abZZZfghijklmnop\"\>"], "Output",
 CellChangeTimes->{3.4657716157326*^9, 3.4658598449946003`*^9, 
  3.468597743083846*^9, 3.468597865984519*^9, 3.4700671794224873`*^9},
 CellLabel->"Out[49]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringReplacePart", "[", 
  RowBox[{"myString", ",", " ", "\"\<ZZZ\>\"", ",", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"3", ",", "5"}], "}"}], ",", " ", 
     RowBox[{"{", 
      RowBox[{"10", ",", "15"}], "}"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[50]:="],

Cell[BoxData["\<\"abZZZfghiZZZp\"\>"], "Output",
 CellChangeTimes->{3.4657716158417997`*^9, 3.4658598450176*^9, 
  3.4685977431301327`*^9, 3.4685978660343933`*^9, 3.47006717946231*^9},
 CellLabel->"Out[50]="]
}, Open  ]],

Cell["Each range can also have its own replacement string.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringReplacePart", "[", 
  RowBox[{"myString", ",", " ", 
   RowBox[{"{", 
    RowBox[{"\"\<ZZZ\>\"", ",", "\"\<WWW\>\""}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"3", ",", "5"}], "}"}], ",", " ", 
     RowBox[{"{", 
      RowBox[{"10", ",", "15"}], "}"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[51]:="],

Cell[BoxData["\<\"abZZZfghiWWWp\"\>"], "Output",
 CellChangeTimes->{3.4657716160446*^9, 3.4658598450396*^9, 
  3.468597743179757*^9, 3.468597866084731*^9, 3.4700671795049067`*^9},
 CellLabel->"Out[51]="]
}, Open  ]],

Cell["Discussion", "Heading2"],

Cell[TextData[{
 StyleBox["As you can see from the given examples, ",
  FontTracking->"Plain"],
 StyleBox["StringReplace", "Literal",
  FontTracking->"Plain"],
 StyleBox[" is quite versatile. However, the versatility is derived from \
Mathematica\[CloseCurlyQuote]s rich support for patterns (see ",
  FontTracking->"Plain"],
 Cell[BoxData[
  FormBox[
   ButtonBox[
    RowBox[{
    "\[OpenCurlyDoubleQuote]", "Introduction", "\[CloseCurlyDoubleQuote]"}],
    BaseStyle->"Hyperlink",
    ButtonData->"IntroductionCh5"], TraditionalForm]]],
 " on page 181",
 StyleBox["). Here are some typical text-processing problems that yield to \
the application of ",
  FontTracking->"Plain"],
 StyleBox["StringReplace[]", "Literal",
  FontTracking->"Plain"],
 StyleBox[" and pattern matching.",
  FontTracking->"Plain"]
}], "Text",
 CellChangeTimes->{{3.470494282239169*^9, 3.470494329780343*^9}, 
   3.470516711888763*^9, {3.4706653517570543`*^9, 3.470665351849881*^9}, {
   3.475588479338489*^9, 3.47558848429786*^9}}],

Cell["Stripping comments", "Heading3",
 CellChangeTimes->{{3.475588490918396*^9, 3.475588491293693*^9}}],

Cell["String expression version:", "Text",
 CellChangeTimes->{{3.470494976364316*^9, 3.470494979519054*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringReplace", "[", 
  RowBox[{
  "\"\<1 + 2 * 3.14 (*precise enough for our purpose*) / 42 (*secret of the \
universe*)\>\"", ",", 
   RowBox[{
    RowBox[{"\"\<(*\>\"", "~~", 
     RowBox[{"ShortestMatch", "[", "___", "]"}], "~~", "\"\<*)\>\""}], 
    "\[Rule]", "\"\<\>\""}]}], "]"}]], "Input",
 CellChangeTimes->{3.470495128447288*^9},
 CellLabel->"In[52]:="],

Cell[BoxData["\<\"1 + 2 * 3.14  / 42 \"\>"], "Output",
 CellChangeTimes->{3.4657716162318*^9, 3.4658598450636*^9, 
  3.468597743233676*^9, 3.468597866130333*^9, 3.470067179542129*^9},
 CellLabel->"Out[52]="]
}, Open  ]],

Cell["Regular expression version:", "Text",
 CellChangeTimes->{{3.470495143705532*^9, 3.470495149718173*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringReplace", "[", 
  RowBox[{
  "\"\<1 + 2 * 3.14 (*precise enough for our purpose*) / 42 (*secret of the \
universe*)\>\"", ",", 
   RowBox[{
    RowBox[{
    "RegularExpression", "[", "\"\<\\\\(\\\\*.*?\\\\*\\\\)\>\"", "]"}], 
    "\[Rule]", "\"\<\>\""}]}], "]"}]], "Input",
 CellChangeTimes->{3.4704951253556643`*^9},
 CellLabel->"In[53]:="],

Cell[BoxData["\<\"1 + 2 * 3.14  / 42 \"\>"], "Output",
 CellChangeTimes->{3.4657716163722*^9, 3.4658598450856*^9, 
  3.4685977432794533`*^9, 3.4685978663634367`*^9, 3.4700671795754623`*^9},
 CellLabel->"Out[53]="]
}, Open  ]],

Cell["Changing delimiters", "Heading3",
 CellChangeTimes->{{3.475588495589332*^9, 3.4755884958617067`*^9}}],

Cell[TextData[{
 "Delimited text (e.g., comma-delimited text) sounds simple at first, but \
many delimited formats allow a way to handle the delimiters as regular text \
by some quoting mechanism, as well as a way to escape quotes themselves. \
Furthermore, you must handle empty fields. If you want to replace a \
comma-delimited format with, say, a semicolon-delimited format, you must \
craft expressions that deal with all cases. Here, ",
 StyleBox["\"\"", "Literal"],
 " is used to escape a double quote. This example does not handle empty \
fields, but see Friedl\[CloseCurlyQuote]s ",
 StyleBox["Mastering Regular Expressions", "Emphasis"],
 " for guidance. "
}], "Text",
 CellChangeTimes->{{3.4657771131066*^9, 3.4657771298176003`*^9}, {
   3.4658561695786*^9, 3.4658562135696*^9}, {3.4658562831456003`*^9, 
   3.4658563013906*^9}, {3.4658564474586*^9, 3.4658564917646*^9}, {
   3.470496335564827*^9, 3.470496373802734*^9}, {3.470497417867983*^9, 
   3.470497447153512*^9}, {3.470497999615839*^9, 3.470498059381773*^9}, 
   3.470498250185319*^9, 3.47049828121986*^9, 3.470516717920918*^9, {
   3.4705768046623363`*^9, 3.470576843134336*^9}, {3.470577040710157*^9, 
   3.470577066431698*^9}, {3.47058142772598*^9, 3.470581434130034*^9}, {
   3.470581491337381*^9, 3.470581496898799*^9}, 3.47558850297416*^9}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
  "delimitedText", " ", "=", " ", 
   "\"\<Ten Thousand,10000, 2710 ,\\\"10,000\\\",\\\"It's \\\"\\\"10 \
Grand\\\"\\\",baby\\\",10k\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"StringJoin", "[", 
  RowBox[{"Riffle", "[", 
   RowBox[{
    RowBox[{"StringCases", "[", 
     RowBox[{"delimitedText", ",", 
      RowBox[{
       RowBox[{
       "RegularExpression", "[", 
        "\"\<([^\\\",]+|\\\"(?:[^\\\"]|\\\"\\\")*\\\")\>\"", "]"}], 
       "\[RuleDelayed]", " ", 
       RowBox[{"StringReplace", "[", 
        RowBox[{"\"\<$1\>\"", ",", 
         RowBox[{
         "\"\<\\\"\\\"\>\"", "\[RuleDelayed]", " ", "\"\<\\\"\>\""}]}], 
        "]"}]}]}], "]"}], ",", "\"\<;\>\""}], "]"}], "]"}]}], "Input",
 CellChangeTimes->{{3.4657729842084*^9, 3.4657730457036*^9}, {
   3.4657731017232*^9, 3.4657731547788*^9}, {3.4657732366632*^9, 
   3.4657732493148003`*^9}, {3.4657733697312*^9, 3.4657733987784*^9}, {
   3.4657734340032*^9, 3.4657734958728*^9}, {3.4657738246584*^9, 
   3.4657738285428*^9}, {3.4657739390064*^9, 3.4657739724528*^9}, {
   3.4657740294708*^9, 3.4657741106844*^9}, {3.4657771385436*^9, 
   3.4657771732256002`*^9}, {3.4657772036275997`*^9, 
   3.4657772670016003`*^9}, {3.4657773555316*^9, 3.4657773748426*^9}, {
   3.4657774061516*^9, 3.4657774592195997`*^9}, 3.4657774926745996`*^9, {
   3.4657775328476*^9, 3.4657775892406*^9}, {3.4657776425646*^9, 
   3.4657777576156*^9}, {3.4658561012335997`*^9, 3.4658561446256*^9}, {
   3.4658562532826*^9, 3.4658562651745996`*^9}, {3.47006659486623*^9, 
   3.470066596286399*^9}, {3.470066632865316*^9, 3.4700666358733263`*^9}},
 CellLabel->"In[54]:="],

Cell[BoxData["\<\"Ten Thousand;10000; 2710 ;\\\"10,000\\\";\\\"It's \\\"10 \
Grand\\\",baby\\\";10k\"\>"], "Output",
 CellChangeTimes->{
  3.4657716165126*^9, 3.4657734987432003`*^9, 3.4657738292292*^9, {
   3.4657739571336*^9, 3.465773973498*^9}, 3.465774031764*^9, 
   3.4657740888132*^9, {3.4657774649546003`*^9, 3.4657774944936*^9}, {
   3.4657775445446*^9, 3.4657775617426*^9}, 3.4657775926076*^9, {
   3.4657776619266*^9, 3.4657777591656*^9}, 3.4658561462626*^9, {
   3.4658562548185997`*^9, 3.4658562664356003`*^9}, 3.4658598452226*^9, 
   3.468597743387553*^9, 3.4685978664096317`*^9, {3.47006661239458*^9, 
   3.470066639224283*^9}, 3.4700671796128073`*^9},
 CellLabel->"Out[55]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Removing XML markup", "Heading3",
 CellChangeTimes->{{3.475588616493402*^9, 3.475588616818581*^9}}],

Cell[TextData[{
 "Simple XML manipulations, such as discarding markup, can be accomplished \
with ",
 StyleBox["StringReplace[]", "Literal"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NotebookDirectory", "[", "]"}]], "Input",
 CellLabel->"In[56]:="],

Cell[BoxData["\<\"/Users/smangano/Documents/workspace/Mathematica \
Cookbook/mathematica/\"\>"], "Output",
 CellChangeTimes->{3.4658572869196*^9, 3.4658598452446003`*^9, 
  3.468597743433213*^9, 3.468597866448748*^9, 3.470067179663014*^9},
 CellLabel->"Out[56]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"xml", " ", "=", " ", 
  RowBox[{"Import", "[", 
   RowBox[{
    RowBox[{"FileNameJoin", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"NotebookDirectory", "[", "]"}], ",", "\"\<..\>\"", ",", 
       "\"\<data\>\"", ",", "\"\<ch02\>\"", ",", "\"\<data1.xml\>\""}], "}"}],
      "]"}], ",", "\"\<Text\>\""}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4658571498606*^9, 3.4658572744306*^9}, {
  3.4658573100875998`*^9, 3.4658573142796*^9}},
 CellLabel->"In[57]:="],

Cell[BoxData["\<\"<?xml version=\\\"1.0\\\" encoding=\\\"UTF-8\\\"?>\\n<!-- \
Some data to use as a test for Mathematica's XML import -->\\n<?test Just for \
didactic purposes?>\\n<data>\\n\\t<item>\\n\\t\\t<name>Leonardo</name>\\n\\t\\\
t<sex>male</sex>\\n\\t\\t<age>8</age>\\n\\t\\t<height>4.7</height>\\n\\t</\
item>\\n\\t<item>\\n\\t\\t<name>Salvatore</name>\\n\\t\\t<sex>male</sex>\\n\\\
t\\t<age>5</age>\\n\\t\\t<height>4.1</height>\\n\\t</item>\\n\\t<item>\\n\\t\\\
t<name>Alexis</name>\\n\\t\\t<sex>female</sex>\\n\\t\\t<age>6</age>\\n\\t\\t<\
height>4.4</height>\\n\\t</item>\\n</data>\\n<!-- Comment at end -->\"\>"], \
"Output",
 CellChangeTimes->{{3.4658572378626003`*^9, 3.4658572758386*^9}, 
   3.4658573152206*^9, 3.4658598452846003`*^9, 3.4685977435251217`*^9, 
   3.468597866518016*^9, 3.4700671798071547`*^9},
 CellLabel->"Out[57]="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringReplace", "[", 
  RowBox[{"xml", ",", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Shortest", "[", 
        RowBox[{"\"\<<\>\"", " ", "~~", " ", 
         RowBox[{
          RowBox[{"Except", "[", "\"\<>\>\"", "]"}], ".."}], "~~", " ", 
         "\"\<>\>\""}], "]"}], " ", ".."}], "\[Rule]", " ", "\"\<\>\""}], ",",
      " ", 
     RowBox[{"Whitespace", " ", "\[Rule]", " ", "\"\< \>\""}]}], "}"}]}], 
  "]"}]], "Input",
 CellLabel->"In[58]:="],

Cell[BoxData["\<\"     Leonardo male 8 4.7   Salvatore male 5 4.1   Alexis \
female 6 4.4   \"\>"], "Output",
 CellChangeTimes->{3.4657716174174*^9, 3.4658573254306*^9, 3.4658598453076*^9,
   3.468597743567637*^9, 3.46859786655103*^9, 3.470067179837894*^9},
 CellLabel->"Out[58]="]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowToolbars->"EditBar",
CellGrouping->Manual,
WindowSize->{916, 698},
WindowMargins->{{362, Automatic}, {Automatic, 30}},
PrintingCopies->1,
PrintingStartingPageNumber->181,
PrintingPageRange->{1, Automatic},
PageHeaders->{{None, None, None}, {None, None, None}},
PageFooters->{{
   Cell[
    TextData[{
      CounterBox["Page"], 
      "\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]Chapter ", 
      CounterBox["ChapterLabel"], ":\[ThickSpace]\[ThickSpace]", 
      "String and Text Processing"}], "Footer"], None, None}, {None, None, 
   Cell[
    TextData[{
      CounterBox[
      "Page", CounterFunction :> (
        Part[{"", 
          "5.0\[ThickSpace]\[ThickSpace]Introduction\[ThickSpace]\[ThickSpace]\
\[ThickSpace]|\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.0\[ThickSpace]\[ThickSpace]Introduction\[ThickSpace]\[ThickSpace]\
\[ThickSpace]|\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.0\[ThickSpace]\[ThickSpace]Introduction\[ThickSpace]\[ThickSpace]\
\[ThickSpace]|\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.0\[ThickSpace]\[ThickSpace]Introduction\[ThickSpace]\[ThickSpace]\
\[ThickSpace]|\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.0\[ThickSpace]\[ThickSpace]Introduction\[ThickSpace]\[ThickSpace]\
\[ThickSpace]|\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.1\[ThickSpace]\[ThickSpace]Comparing Strings\[ThickSpace]\
\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.2\[ThickSpace]\[ThickSpace]Removing and Replacing Characters \
\[LineSeparator]from Strings\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.2\[ThickSpace]\[ThickSpace]Removing and Replacing Characters \
\[LineSeparator]from Strings\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.2\[ThickSpace]\[ThickSpace]Removing and Replacing Characters \
\[LineSeparator]from Strings\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.2\[ThickSpace]\[ThickSpace]Removing and Replacing Characters \
\[LineSeparator]from Strings\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.3\[ThickSpace]\[ThickSpace]Extracting Characters and Substrings\
\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]", 
          "5.3\[ThickSpace]\[ThickSpace]Extracting Characters and Substrings\
\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]", 
          "5.3\[ThickSpace]\[ThickSpace]Extracting Characters and Substrings\
\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]", 
          "5.3\[ThickSpace]\[ThickSpace]Extracting Characters and Substrings\
\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]", 
          "5.4\[ThickSpace]\[ThickSpace]Duplicating a String \[ThickSpace]\
\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.4\[ThickSpace]\[ThickSpace]Duplicating a String \[ThickSpace]\
\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.5\[ThickSpace]\[ThickSpace]Matching and Searching Text\
\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]", 
          "5.5\[ThickSpace]\[ThickSpace]Matching and Searching Text\
\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]", 
          "5.5\[ThickSpace]\[ThickSpace]Matching and Searching Text\
\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]", 
          "5.6\[ThickSpace]\[ThickSpace]Tokenizing Text\[ThickSpace]\
\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.7\[ThickSpace]\[ThickSpace]Working with Natural Language \
Dictionaries\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]", 
          "5.7\[ThickSpace]\[ThickSpace]Working with Natural Language \
Dictionaries\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]", 
          "5.7\[ThickSpace]\[ThickSpace]Working with Natural Language \
Dictionaries\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]", 
          "5.7\[ThickSpace]\[ThickSpace]Working with Natural Language \
Dictionaries\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]", 
          "5.7\[ThickSpace]\[ThickSpace]Working with Natural Language \
Dictionaries\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]", 
          "5.7\[ThickSpace]\[ThickSpace]Working with Natural Language \
Dictionaries\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]", 
          "5.7\[ThickSpace]\[ThickSpace]Working with Natural Language \
Dictionaries\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]", 
          "5.8\[ThickSpace]\[ThickSpace]Importing XML \[ThickSpace]\
\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.8\[ThickSpace]\[ThickSpace]Importing XML \[ThickSpace]\
\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.8\[ThickSpace]\[ThickSpace]Importing XML \[ThickSpace]\
\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.8\[ThickSpace]\[ThickSpace]Importing XML \[ThickSpace]\
\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.9\[ThickSpace]\[ThickSpace]Transforming XML Using Patterns and \
Rules\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]", 
          "5.9\[ThickSpace]\[ThickSpace]Transforming XML Using Patterns and \
Rules\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]", 
          "5.9\[ThickSpace]\[ThickSpace]Transforming XML Using Patterns and \
Rules\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]", 
          "5.9\[ThickSpace]\[ThickSpace]Transforming XML Using Patterns and \
Rules\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]", 
          "5.9\[ThickSpace]\[ThickSpace]Transforming XML Using Patterns and \
Rules\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]", 
          "5.10\[ThickSpace]\[ThickSpace]Transforming XML Using Recursive \
Functions (\[AGrave] la XSLT)\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.10\[ThickSpace]\[ThickSpace]Transforming XML Using Recursive \
Functions (\[AGrave] la XSLT)\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.10\[ThickSpace]\[ThickSpace]Transforming XML Using Recursive \
Functions (\[AGrave] la XSLT)\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.10\[ThickSpace]\[ThickSpace]Transforming XML Using Recursive \
Functions (\[AGrave] la XSLT)\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.10\[ThickSpace]\[ThickSpace]Transforming XML Using Recursive \
Functions (\[AGrave] la XSLT)\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.10\[ThickSpace]\[ThickSpace]Transforming XML Using Recursive \
Functions (\[AGrave] la XSLT)\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.10\[ThickSpace]\[ThickSpace]Transforming XML Using Recursive \
Functions (\[AGrave] la XSLT)\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.10\[ThickSpace]\[ThickSpace]Transforming XML Using Recursive \
Functions (\[AGrave] la XSLT)\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.10\[ThickSpace]\[ThickSpace]Transforming XML Using Recursive \
Functions (\[AGrave] la XSLT)\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.11\[ThickSpace]\[ThickSpace]Writing Parsers and Grammars \
\[LineSeparator]in Mathematica\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.11\[ThickSpace]\[ThickSpace]Writing Parsers and Grammars \
\[LineSeparator]in Mathematica\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.11\[ThickSpace]\[ThickSpace]Writing Parsers and Grammars \
\[LineSeparator]in Mathematica\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.11\[ThickSpace]\[ThickSpace]Writing Parsers and Grammars \
\[LineSeparator]in Mathematica\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.11\[ThickSpace]\[ThickSpace]Writing Parsers and Grammars \
\[LineSeparator]in Mathematica\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.11\[ThickSpace]\[ThickSpace]Writing Parsers and Grammars \
\[LineSeparator]in Mathematica\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.11\[ThickSpace]\[ThickSpace]Writing Parsers and Grammars \
\[LineSeparator]in Mathematica\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.11\[ThickSpace]\[ThickSpace]Writing Parsers and Grammars \
\[LineSeparator]in Mathematica\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]", 
          "5.11\[ThickSpace]\[ThickSpace]Writing Parsers and Grammars \
\[LineSeparator]in Mathematica\[ThickSpace]\[ThickSpace]\[ThickSpace]|\
\[ThickSpace]\[ThickSpace]\[ThickSpace]"}, # - CurrentValue[
          InputNotebook[], PrintingStartingPageNumber] + 1]& )], 
      CounterBox["Page"]}], "Footer"]}},
PageHeaderLines->{False, False},
PageFooterLines->{True, True},
PrintingOptions->{"CellBackgroundHalftoneAngle"->Automatic,
"CellBackgroundHalftoneDensity"->Automatic,
"FacingPages"->True,
"FirstPageFace"->"Right",
"FirstPageFooter"->True,
"FirstPageHeader"->False,
"GraphicsPrintingFormat"->"Automatic",
"IncludePostScriptResourceDirectives"->True,
"IncludeSpecialFonts"->True,
"Magnification"->1.,
"OpacityRenderingMethod"->Automatic,
"PageFooterMargins"->{24, 24},
"PageHeaderMargins"->{36, 36},
"PageSize"->{504, 661.5},
"PaperOrientation"->"Portrait",
"PaperSize"->{Automatic, Automatic},
"PostScriptOutputFile"->Automatic,
"PrintCellBrackets"->False,
"PrintMultipleHorizontalPages"->False,
"PrintRegistrationMarks"->True,
"PrintSelectionHighlighting"->False,
"PrintingMargins"->{{72, 72}, {72, 54}},
"RasterizationResolution"->"Automatic",
"RestPagesFooter"->True,
"RestPagesHeader"->True,
"UnixShellPrintingCommand"->Automatic,
"UsePostScriptOutputFile"->False,
"UseUnixShellPrintingCommand"->False,
"VertexColorRenderingMethod"->Automatic},
ShowSelection->True,
CellLabelAutoDelete->False,
Magnification:>FEPrivate`If[
  FEPrivate`Equal[FEPrivate`$VersionNumber, 6.], 1.25, 1.25 Inherited],
FrontEndVersion->"7.0 for Mac OS X x86 (32-bit) (2009\:5e744\:670823\:65e5)",
StyleDefinitions->"ora.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "chStrings"->{
  Cell[545, 20, 146, 6, 63, "ChapterLabel",
   CounterAssignments->{{"ChapterLabel", 5}},
   CellTags->"chStrings"]},
 "IntroductionCh5"->{
  Cell[1551, 58, 172, 9, 63, "Heading1",
   CellTags->"IntroductionCh5"]},
 "tStringPatterns"->{
  Cell[20244, 629, 152, 2, 34, "TableTitle",
   CellTags->"tStringPatterns"]},
 "tRegExPatterns"->{
  Cell[23842, 724, 177, 3, 17, "TableTitle",
   CellTags->"tRegExPatterns"]},
 "ComapringStrings"->{
  Cell[28503, 842, 201, 9, 63, "Heading1",
   CellTags->"ComapringStrings"]},
 "Removing and Replacing Characters from Strings"->{
  Cell[32978, 1006, 330, 11, 63, "Heading1",
   CellTags->"Removing and Replacing Characters from Strings"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"chStrings", 66626, 1890},
 {"IntroductionCh5", 66766, 1894},
 {"tStringPatterns", 66863, 1897},
 {"tRegExPatterns", 66963, 1900},
 {"ComapringStrings", 67064, 1903},
 {"Removing and Replacing Characters from Strings", 67195, 1906}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[545, 20, 146, 6, 63, "ChapterLabel",
 CounterAssignments->{{"ChapterLabel", 5}},
 CellTags->"chStrings"],
Cell[694, 28, 50, 0, 63, "ChapterTitle"],
Cell[747, 30, 561, 17, 210, "Epigraph"],
Cell[1311, 49, 215, 5, 60, "EpigraphAuthor"],
Cell[CellGroupData[{
Cell[1551, 58, 172, 9, 63, "Heading1",
 CellTags->"IntroductionCh5"],
Cell[1726, 69, 595, 9, 82, "Text"],
Cell[2324, 80, 232, 5, 25, "Text"],
Cell[CellGroupData[{
Cell[2581, 89, 54, 0, 75, "Heading2"],
Cell[2638, 91, 313, 7, 44, "Text"],
Cell[2954, 100, 404, 11, 44, "Text"],
Cell[CellGroupData[{
Cell[3383, 115, 101, 2, 49, "Input"],
Cell[3487, 119, 203, 3, 51, "Output"]
}, Open  ]],
Cell[3705, 125, 180, 5, 25, "Text"],
Cell[CellGroupData[{
Cell[3910, 134, 182, 5, 49, "Input"],
Cell[4095, 141, 202, 3, 54, "Output"]
}, Open  ]],
Cell[4312, 147, 515, 12, 63, "Text"],
Cell[CellGroupData[{
Cell[4852, 163, 67, 1, 49, "Input"],
Cell[4922, 166, 200, 3, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5159, 174, 73, 1, 49, "Input"],
Cell[5235, 177, 193, 3, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5465, 185, 162, 5, 53, "Input"],
Cell[5630, 192, 2522, 52, 304, "Output"]
}, Open  ]],
Cell[8167, 247, 159, 2, 25, "Text"],
Cell[CellGroupData[{
Cell[8351, 253, 181, 5, 49, "Input"],
Cell[8535, 260, 221, 3, 54, "Output"]
}, Open  ]],
Cell[8771, 266, 174, 4, 25, "Text"],
Cell[CellGroupData[{
Cell[8970, 274, 225, 6, 49, "Input"],
Cell[9198, 282, 233, 5, 53, "Output"]
}, Open  ]],
Cell[9446, 290, 163, 4, 25, "Text"],
Cell[CellGroupData[{
Cell[9634, 298, 164, 4, 48, "Input"],
Cell[9801, 304, 189, 3, 51, "Output"]
}, Open  ]],
Cell[10005, 310, 72, 0, 25, "Text"],
Cell[CellGroupData[{
Cell[10102, 314, 188, 6, 48, "Input"],
Cell[10293, 322, 201, 3, 51, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[10543, 331, 50, 0, 75, "Heading2"],
Cell[10596, 333, 2100, 42, 194, "Text"],
Cell[CellGroupData[{
Cell[12721, 379, 85, 1, 75, "Heading3"],
Cell[12809, 382, 1048, 30, 63, "Text"],
Cell[13860, 414, 101, 4, 25, "Text"],
Cell[13964, 420, 111, 3, 49, "Input"],
Cell[CellGroupData[{
Cell[14100, 427, 176, 4, 49, "Input"],
Cell[14279, 433, 184, 3, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[14500, 441, 177, 4, 49, "Input"],
Cell[14680, 447, 187, 3, 51, "Output"]
}, Open  ]],
Cell[14882, 453, 133, 4, 25, "Text"],
Cell[15018, 459, 112, 3, 49, "Input"],
Cell[CellGroupData[{
Cell[15155, 466, 178, 4, 49, "Input"],
Cell[15336, 472, 181, 3, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15554, 480, 176, 4, 49, "Input"],
Cell[15733, 486, 190, 3, 51, "Output"]
}, Open  ]],
Cell[15938, 492, 111, 4, 25, "Text"],
Cell[16052, 498, 113, 3, 49, "Input"],
Cell[CellGroupData[{
Cell[16190, 505, 178, 4, 49, "Input"],
Cell[16371, 511, 184, 3, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[16592, 519, 177, 4, 49, "Input"],
Cell[16772, 525, 183, 3, 51, "Output"]
}, Open  ]],
Cell[16970, 531, 259, 4, 44, "Text"],
Cell[CellGroupData[{
Cell[17254, 539, 184, 4, 49, "Input"],
Cell[17441, 545, 192, 3, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[17670, 553, 258, 5, 49, "Input"],
Cell[17931, 560, 189, 3, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[18157, 568, 277, 7, 49, "Input"],
Cell[18437, 577, 189, 3, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[18663, 585, 275, 7, 49, "Input"],
Cell[18941, 594, 190, 3, 51, "Output"]
}, Open  ]],
Cell[19146, 600, 1095, 27, 63, "Text"],
Cell[20244, 629, 152, 2, 34, "TableTitle",
 CellTags->"tStringPatterns"],
Cell[20399, 633, 2049, 47, 430, "Table",
 Evaluatable->False],
Cell[22451, 682, 1364, 38, 84, "Text"]
}, Open  ]]
}, Open  ]],
Cell[23842, 724, 177, 3, 17, "TableTitle",
 CellTags->"tRegExPatterns"],
Cell[24022, 729, 2305, 51, 432, "Table",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[26352, 784, 61, 1, 28, "Heading2"],
Cell[26416, 787, 852, 16, 63, "Text"]
}, Open  ]]
}, Open  ]],
Cell[27295, 807, 1205, 33, 63, "Text"],
Cell[28503, 842, 201, 9, 63, "Heading1",
 CellTags->"ComapringStrings"],
Cell[CellGroupData[{
Cell[28729, 855, 40, 0, 46, "Heading2WithHeadAbove"],
Cell[28772, 857, 242, 10, 25, "Text"]
}, Open  ]],
Cell[29029, 870, 28, 0, 75, "Heading2"],
Cell[29060, 872, 261, 7, 24, "Text"],
Cell[CellGroupData[{
Cell[29346, 883, 126, 3, 48, "Input"],
Cell[29475, 888, 185, 3, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[29697, 896, 125, 3, 48, "Input"],
Cell[29825, 901, 200, 4, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[30062, 910, 30, 0, 75, "Heading2"],
Cell[30095, 912, 453, 10, 63, "Text"],
Cell[30551, 924, 1415, 43, 178, "Input"],
Cell[CellGroupData[{
Cell[31991, 971, 103, 2, 46, "Input"],
Cell[32097, 975, 185, 3, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[32319, 983, 103, 2, 46, "Input"],
Cell[32425, 987, 194, 3, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[32656, 995, 104, 2, 46, "Input"],
Cell[32763, 999, 188, 3, 51, "Output"]
}, Open  ]]
}, Open  ]],
Cell[32978, 1006, 330, 11, 63, "Heading1",
 CellTags->"Removing and Replacing Characters from Strings"],
Cell[CellGroupData[{
Cell[33333, 1021, 40, 0, 46, "Heading2WithHeadAbove"],
Cell[33376, 1023, 358, 8, 44, "Text"]
}, Open  ]],
Cell[33749, 1034, 61, 1, 33, "Heading2"],
Cell[CellGroupData[{
Cell[33835, 1039, 89, 1, 46, "Heading3WithHeadAbove"],
Cell[33927, 1042, 780, 22, 44, "Text"],
Cell[34710, 1066, 65, 1, 21, "Text"],
Cell[34778, 1069, 216, 7, 40, "Input"],
Cell[CellGroupData[{
Cell[35019, 1080, 180, 4, 49, "Input"],
Cell[35202, 1086, 231, 3, 54, "Output"]
}, Open  ]],
Cell[35448, 1092, 86, 1, 21, "Text"],
Cell[CellGroupData[{
Cell[35559, 1097, 304, 8, 44, "Input"],
Cell[35866, 1107, 308, 5, 51, "Output"]
}, Open  ]],
Cell[36189, 1115, 204, 5, 18, "Text"],
Cell[CellGroupData[{
Cell[36418, 1124, 496, 14, 66, "Input"],
Cell[36917, 1140, 311, 5, 51, "Output"]
}, Open  ]],
Cell[37243, 1148, 76, 1, 18, "Text"],
Cell[CellGroupData[{
Cell[37344, 1153, 253, 7, 40, "Input"],
Cell[37600, 1162, 312, 5, 51, "Output"]
}, Open  ]],
Cell[37927, 1170, 75, 1, 18, "Text"],
Cell[CellGroupData[{
Cell[38027, 1175, 311, 8, 40, "Input"],
Cell[38341, 1185, 308, 5, 51, "Output"]
}, Open  ]],
Cell[38664, 1193, 190, 4, 16, "Text"],
Cell[CellGroupData[{
Cell[38879, 1201, 245, 7, 40, "Input"],
Cell[39127, 1210, 250, 4, 54, "Output"]
}, Open  ]]
}, Open  ]],
Cell[39404, 1218, 124, 1, 75, "Heading3"],
Cell[39531, 1221, 452, 9, 62, "Text"],
Cell[39986, 1232, 25, 0, 25, "Text"],
Cell[40014, 1234, 139, 4, 49, "Input"],
Cell[40156, 1240, 58, 0, 25, "Text"],
Cell[CellGroupData[{
Cell[40239, 1244, 122, 3, 49, "Input"],
Cell[40364, 1249, 204, 3, 54, "Output"]
}, Open  ]],
Cell[40583, 1255, 141, 1, 25, "Text"],
Cell[CellGroupData[{
Cell[40749, 1260, 141, 4, 49, "Input"],
Cell[40893, 1266, 203, 3, 54, "Output"]
}, Open  ]],
Cell[41111, 1272, 99, 1, 25, "Text"],
Cell[CellGroupData[{
Cell[41235, 1277, 146, 4, 49, "Input"],
Cell[41384, 1283, 209, 3, 54, "Output"]
}, Open  ]],
Cell[41608, 1289, 193, 5, 25, "Text"],
Cell[CellGroupData[{
Cell[41826, 1298, 171, 5, 49, "Input"],
Cell[42000, 1305, 200, 3, 54, "Output"],
Cell[42203, 1310, 424, 11, 63, "Text"],
Cell[CellGroupData[{
Cell[42652, 1325, 197, 6, 49, "Input"],
Cell[42852, 1333, 196, 3, 54, "Output"]
}, Open  ]]
}, Open  ]],
Cell[43075, 1340, 60, 0, 25, "Text"],
Cell[43138, 1342, 131, 4, 49, "Input"],
Cell[CellGroupData[{
Cell[43294, 1350, 235, 7, 49, "Input"],
Cell[43532, 1359, 265, 6, 54, "Output"]
}, Open  ]],
Cell[43812, 1368, 437, 10, 44, "Text"],
Cell[CellGroupData[{
Cell[44274, 1382, 203, 5, 49, "Input"],
Cell[44480, 1389, 210, 3, 54, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[44727, 1397, 316, 9, 49, "Input"],
Cell[45046, 1408, 208, 3, 54, "Output"]
}, Open  ]],
Cell[45269, 1414, 68, 0, 25, "Text"],
Cell[CellGroupData[{
Cell[45362, 1418, 375, 11, 49, "Input"],
Cell[45740, 1431, 203, 3, 54, "Output"]
}, Open  ]],
Cell[45958, 1437, 30, 0, 75, "Heading2"],
Cell[45991, 1439, 1009, 26, 62, "Text"],
Cell[47003, 1467, 104, 1, 75, "Heading3"],
Cell[47110, 1470, 108, 1, 24, "Text"],
Cell[CellGroupData[{
Cell[47243, 1475, 388, 10, 92, "Input"],
Cell[47634, 1487, 207, 3, 53, "Output"]
}, Open  ]],
Cell[47856, 1493, 109, 1, 25, "Text"],
Cell[CellGroupData[{
Cell[47990, 1498, 371, 10, 93, "Input"],
Cell[48364, 1510, 213, 3, 53, "Output"]
}, Open  ]],
Cell[48592, 1516, 107, 1, 75, "Heading3"],
Cell[48702, 1519, 1315, 21, 99, "Text"],
Cell[CellGroupData[{
Cell[50042, 1544, 1653, 34, 121, "Input"],
Cell[51698, 1580, 690, 11, 54, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[52425, 1596, 105, 1, 75, "Heading3"],
Cell[52533, 1599, 158, 5, 25, "Text"],
Cell[CellGroupData[{
Cell[52716, 1608, 89, 2, 49, "Input"],
Cell[52808, 1612, 263, 4, 54, "Output"]
}, Open  ]],
Cell[53086, 1619, 493, 12, 49, "Input"],
Cell[53582, 1633, 851, 12, 525, "Output"],
Cell[CellGroupData[{
Cell[54458, 1649, 511, 15, 49, "Input"],
Cell[54972, 1666, 281, 4, 51, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
